"use strict";

var _console = require("@resdir/console");

var _httpClient = require("@resdir/http-client");

var _localCache = _interopRequireDefault(require("@resdir/local-cache"));

var _version = require("@resdir/version");

var _ = require("../");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const INSTALL_WEBSITE_URL = 'https://install.run.tools';
const INSTALL_COMMAND = `curl ${INSTALL_WEBSITE_URL} | bash`;
const LATEST_VERSION_CACHE_TIME = 3 * 24 * 60 * 60 * 1000; // 3 days

function start() {
  return _start.apply(this, arguments);
}

function _start() {
  _start = _asyncToGenerator(function* () {
    let expression = process.argv.slice(2);
    let printOutput;
    let index = expression.indexOf('--@print');

    if (index === -1) {
      index = expression.indexOf('--@p');
    }

    if (index !== -1) {
      printOutput = true;
      expression.splice(index, 1);
    }

    expression = expression.map(arg => arg.replace(/\s/g, '\\ '));
    expression = expression.join(' ');
    const directory = process.cwd();

    if (expression.includes('@version')) {
      // TODO: move this in run-core
      (0, _console.print)((yield getCurrentVersion()));
      return;
    }

    if (expression.includes('@repl')) {
      // TODO: move this in resource/helper
      yield (0, _.runREPL)({
        directory
      });
      return;
    }

    const output = yield (0, _.runExpression)(expression, {
      directory
    });

    if (printOutput) {
      output.$print();
    } else if (output && !output.$getIsMethodOutput()) {
      yield output['@help']();
    }
  });
  return _start.apply(this, arguments);
}

function checkVersion() {
  return _checkVersion.apply(this, arguments);
}

function _checkVersion() {
  _checkVersion = _asyncToGenerator(function* () {
    const currentVersion = yield getCurrentVersion();
    const latestVersion = yield getLatestVersion();

    if (!latestVersion) {
      return;
    }

    if ((0, _version.compareVersions)(currentVersion, '<', latestVersion)) {
      (0, _console.print)(`${(0, _console.formatBold)(`Run CLI update available ${(0, _console.formatDim)(`(${currentVersion} → ${latestVersion})`)}`)}, invoke ${(0, _console.formatCode)(INSTALL_COMMAND)} to update.`);
    }
  });
  return _checkVersion.apply(this, arguments);
}

function getCurrentVersion() {
  return _getCurrentVersion.apply(this, arguments);
}

function _getCurrentVersion() {
  _getCurrentVersion = _asyncToGenerator(function* () {
    const pkg = require('../../../../package.json');

    return pkg.version;
  });
  return _getCurrentVersion.apply(this, arguments);
}

function getLatestVersion() {
  return _getLatestVersion.apply(this, arguments);
}

function _getLatestVersion() {
  _getLatestVersion = _asyncToGenerator(function* () {
    try {
      const cache = new _localCache.default({
        time: LATEST_VERSION_CACHE_TIME
      });
      const {
        body
      } = yield (0, _httpClient.get)(`${INSTALL_WEBSITE_URL}/releases/latest.txt`, {
        cache,
        timeout: 5000
      });
      return body.trim();
    } catch (err) {
      (0, _console.print)(`${(0, _console.formatDanger)('An error occurred while checking the latest version')} ${(0, _console.formatDim)(`(${err.message})`)}`);
      return undefined;
    }
  });
  return _getLatestVersion.apply(this, arguments);
}

_asyncToGenerator(function* () {
  yield checkVersion();
  yield start();
})().catch(_console.printErrorAndExit);